@@ -3,7 +3,7 @@ context("ipriorMod methods")
 test_that("Methods for regular fit",{
 
   mod <- iprior(stack.loss ~ ., data = stackloss, control = list(silent = TRUE))
-  res1 <- predict(mod)
+  res1 <- fitted(mod)
   res2 <- predict(mod, stackloss[1:10, ])
   expect_equivalent(res1[1:10], res2)
   expect_equal(mod$log.lik, logLik(mod))
@@ -14,7 +14,7 @@ test_that("Methods for regular fit",{
 test_that("Methods for interactions fit",{
 
   mod <- iprior(len ~ . ^ 2, data = ToothGrowth, control = list(silent = TRUE))
-  res1 <- predict(mod)
+  res1 <- fitted(mod)
   res2 <- predict(mod, ToothGrowth[c(9, 39, 59), ])
   expect_equivalent(res1[c(9, 39, 59)], res2)
   expect_equal(mod$log.lik, logLik(mod))
@@ -27,7 +27,7 @@ test_that("Methods for one.lam = TRUE fit",{
   data(pollution)
   mod <- iprior(Mortality ~ ., data = pollution, model = list(one.lam = TRUE),
                 control = list(silent = TRUE))
-  res1 <- predict(mod)
+  res1 <- fitted(mod)
   x <- sample(1:60, 10, replace = FALSE)
   res2 <- predict(mod, pollution[x, ])
   expect_equivalent(res1[x], res2)
@@ -41,7 +41,7 @@ test_that("Methods for higher order fit",{
   mod <- iprior(stack.loss ~ Air.Flow + I(Air.Flow ^ 2) + ., data = stackloss,
                 model = list(order = c(1, "1^2", 2, 3)),
                 control = list(silent = TRUE, maxit = 5))
-  res1 <- predict(mod)
+  res1 <- fitted(mod)
   res2 <- predict(mod, stackloss[1:10, ])
   expect_equivalent(res1[1:10], res2)
   expect_equal(mod$log.lik, logLik(mod))
